/*
 * CODENVY CONFIDENTIAL
 * __________________
 *
 * [2012] - [2013] Codenvy, S.A.
 * All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Codenvy S.A. and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Codenvy S.A.
 * and its suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Codenvy S.A..
 */
package org.exoplatform.ide.git.client.branch;

import com.google.gwt.cell.client.SafeHtmlCell;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.user.cellview.client.CellTable;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Image;

import org.exoplatform.gwtframework.ui.client.component.ListGrid;
import org.exoplatform.ide.git.client.GitClientBundle;
import org.exoplatform.ide.git.client.GitExtension;
import org.exoplatform.ide.git.shared.Branch;

import java.util.List;

/**
 * Grid for disaplying branches.
 * 
 * @author <a href="mailto:zhulevaanna@gmail.com">Ann Zhuleva</a>
 * @version $Id: Apr 8, 2011 11:31:09 AM anya $
 */
public class BranchGrid extends ListGrid<Branch> {
    /** Grid's ID. */
    private static final String ID   = "ideBranchGrid";

    /** Name column's name. */
    private final String        NAME = GitExtension.MESSAGES.branchGridNameColumn();

    /** Name column. */
    Column<Branch, SafeHtml>    nameColumn;

    public BranchGrid() {
        super();
        setID(ID);
        initColumns();
    }

    /** Initialize the colums of the grid. */
    private void initColumns() {
        CellTable<Branch> cellTable = getCellTable();

        nameColumn = new Column<Branch, SafeHtml>(new SafeHtmlCell()) {

            @Override
            public SafeHtml getValue(final Branch branch) {
                SafeHtml html = new SafeHtml() {
                    private static final long serialVersionUID = 1L;

                    @Override
                    public String asString() {
                        if (branch.isActive()) {
                            return branch.getDisplayName() + "&nbsp;" + new Image(GitClientBundle.INSTANCE.currentBranch());
                        } else {
                            return branch.getDisplayName();
                        }
                    }
                };
                return html;
            }

        };

        nameColumn.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_LEFT);
        cellTable.addColumn(nameColumn, NAME);
        cellTable.setColumnWidth(nameColumn, 35, Unit.PCT);
    }

    /** @see org.exoplatform.gwtframework.ui.client.component.ListGrid#setValue(java.util.List) */
    @Override
    public void setValue(List<Branch> value) {
        super.setValue(value);
        if (value != null && value.size() > 0) {
            selectItem(value.get(0));
            updateGrid();
        }
    }

    /**
     * Returns selected branch in branches grid.
     * 
     * @return {@link Branch} selected branch
     */
    public Branch getSelectedBranch() {
        return super.getSelectedItems().get(0);
    }
}
